;; wii-disc.scm -*-scheme-*-
;; Create a wii disc with transparancy for usb loader gx
;; http://wii2.fr
;; Version 1.1
;;
;; Copyright wii2.fr
;;
;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (script-fu-wii-disc inImage
                            inLayer
                            inFileName
                            inDestination
                            inSaveName
                            inCentralHole
                            inHoleRatio
                            inFinalSize
                            inAddForeGround
        )

    (
        let* (
                ; Original width
                (width (car (gimp-image-width inImage)))
                (HoleWidth (/ width inHoleRatio))
                (holePos (- (/ width 2) (/ HoleWidth 2)))
                
                ; Load picture into variable "layer"
                (layer  (car (gimp-file-load-layer 0 inImage inFileName)))
                
                ; Create save file name
                (saveFile (string-append inDestination "/" inSaveName ".png"))
                
                ; Used to get the square's size
                (size 0) 
                (tmp 0) 
                
                ; copy actual layer to restore after treatment
                (undoLayer (car (gimp-layer-copy inLayer 1)))
            )
        
        ; Create an "Undo" group
        (gimp-image-undo-group-start inImage)
        
        ; Release selection
        (gimp-selection-none inImage)
        
        ; Add layer to current image
        (gimp-image-add-layer inImage layer -1)
        
        ; Picture => square
        (set! size (car (gimp-drawable-width layer)))
        (set! tmp  (car (gimp-drawable-height layer)))
        (if (= size tmp)
            ()
            (begin
                (if (> size tmp) (set! size tmp))
                (gimp-layer-resize layer size size 0 0)
            )
        )
        
        ; Resize layer
        (gimp-layer-scale layer width width 1)
    	(gimp-layer-set-offsets layer 0 0)
    	
        ; Add transparancy
        (gimp-layer-add-alpha layer)
        
        ; Centered hole
        (if (= inCentralHole TRUE)
            (begin
                (gimp-ellipse-select inImage holePos holePos HoleWidth HoleWidth 0 1 0 0)
                (gimp-edit-clear layer)
            )
        )
        
        ; Disc border
        (gimp-ellipse-select inImage 0 0 width width 0 1 0 0)
        (gimp-selection-invert inImage)
        (gimp-edit-clear layer)
        
        (if (= inAddForeGround TRUE)
            (begin
                ; Set layer under the disk custom label
                (gimp-image-lower-layer-to-bottom inImage layer)
                
                ; Merge layers
                (gimp-image-merge-visible-layers inImage 1)
            )
            (begin
            	; Remove treated layer
            	(gimp-image-remove-layer inImage inLayer)
            )
        )
        
        ; Resize picture
            ; USB Loader need "4 divisable" size 
        (set! inFinalSize (- inFinalSize (modulo inFinalSize 4)))
        (set! layer (car (gimp-image-get-active-layer inImage)))
        (gimp-layer-scale layer inFinalSize inFinalSize 1)
        (gimp-image-resize-to-layers inImage)
        
        ; Save PNG
    	(file-png-save 1 inImage layer saveFile saveFile 0 0 0 0 0 1 1)
    	
    	; Remove treated layer
    	(gimp-image-remove-layer inImage layer)
    	
    	; Add original layer
        (gimp-image-add-layer inImage undoLayer -1)
        
        ; Resize image to original layer
        (gimp-image-resize-to-layers inImage)
        
        ; Refresh display
    	(gimp-displays-flush)
    	
        ; Close the "Undo" group
        (gimp-image-undo-group-end inImage)
        
    )
)


(script-fu-register 
    "script-fu-wii-disc"
    "Disc"
    "Create a transparancy disc for USB Loader Gx (and others)"
    "Chpoun's"
    "Pas Glop !"
    "2009/06/15"
    ""
    SF-IMAGE    "Image" 0
    SF-DRAWABLE "Drawable" 0
    SF-FILENAME "Image" "/"
    SF-DIRNAME  "Save directory" ""
    SF-STRING   "Save file (without extension)" ""
    SF-TOGGLE   "Central hole" 1
    SF-VALUE    "Hole ratio (x time smaller than disc)" "8"
    SF-VALUE    "Finale size (px)" "160"
    SF-TOGGLE   "Add foreground" 1
    )

(script-fu-menu-register "script-fu-wii-disc" "<Image>/Filters/Wii")